;;This file is part of Project Thing.
;;
;;Project Thing is free software: you can redistribute it and/or modify
;;it under the terms of the GNU General Public License as published by
;;the Free Software Foundation, either version 3 of the License, or
;;(at your option) any later version.
;;
;;Project Thing is distributed in the hope that it will be useful,
;;but WITHOUT ANY WARRANTY; without even the implied warranty of
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;GNU General Public License for more details.
;;
;;You should have received a copy of the GNU General Public License
;;along with Project Thing.  If not, see <https://www.gnu.org/licenses/>.

;;
;;Enemy AI
;;

.include "ai_control_system.h.s"


bombPtr: .dw 0x0000 
ai_delay: .db #_ai_delay_default
standby_delay: .db #_ai_standby_delay

;;Inicializa el control por inteligencia artificial
;;  INPUT:: IX puntero a el vector de entidades ia
ai_control_init::
    ld (_entity_array_ptr), ix
    ld (_entity_player), ix

    ld de, #sizeof_e
    add ix, de
    ld (bombPtr), ix

    ret

set_standby_delay_one::
    
    ld a, #1
    ld (standby_delay), a

    ret

;;The ai will wait for the player to start moving to seek him out
ai_control_stand_by::
    ld a, (standby_delay)
    dec a
    ld (standby_delay), a
    ret nz
    ld a, #_ai_standby_delay
    ld (standby_delay), a
    ld iy, (bombPtr)
    ld a, e_y(ix)
    ld e_y(iy), a
    ld e_vy(iy), #0
    ld e_ai_state(iy), #ai_bomb_held
    ld e_ai_state(ix), #ai_seek

    ld e_state(iy), #0
    
    ret

;;The ai will modify it's speed to move towards the player, once the enemy's x position is equal to the player's the ai state will be changed to destroy
ai_control_seek::
    _entity_player = . + 2
    ld iy, #0x0000

    ;;compara las posiciones x del jugador y la entidad ia y modifica la velocidad de la entidad ia para seguir al jugador
    ld a, e_x(ix)
    ld b, e_x(iy)
    cp b
    jr z, _no_move_necesary
    ;;si a es menor que b
    jr c, _move_right
    ;;si a es mayor que b
    jr _move_left

_move_right:
    ld e_vx(ix), #1
    jr _end_if_seek

_move_left:
    ld e_vx(ix), #-1
    jr _end_if_seek

_no_move_necesary:
    ld e_vx(ix), #0
    ld e_ai_state(ix), #ai_destroy

    ld e_state(ix), #enemy_noIA_bomb

_end_if_seek:
    ;; Modifica la posicion de la bomba para que este siempre detras del enemigo
    ld iy, (bombPtr)
    ld a, e_x(ix)
    inc a
    ld e_x(iy), a
    ld a, e_y(ix)
    ld e_y(iy), a
    ret

;;The ai will drop a bomb on the player and jump to stanby state
ai_control_destroy::
    ;;implementar comportamiento de soltar bomba
    ;;Poner bomb en la misma posicion del enemigo y convertirla en no ia
    ld iy, (bombPtr)
    ld a, e_x(ix)
    inc a
    ld e_x(iy), a
    ld a, e_y(ix)
    ld e_y(iy), a
    ld e_ai_state(iy), #ai_no_ai
    ld e_ai_state(ix), #ai_stand_by

    ld e_state(iy), #enemy_noIA_bomb

    ret


;;Actualiza las entidades ia 
;;  INPUT:: A numero de entidades en el array de entidades.
ai_control_update::

    ld (_ent_counter), a
    ;; se almacena en ix el valor del puntrero a el array de entidades
    _entity_array_ptr = . + 2
    ld ix, #0x0000

    ld a, e_draw(ix)
    cp #on

    jr nz, end_control_update 

    ;;loop to decrease delay until 0, then execute update and reset delay
    ld a, (ai_delay)
    dec a
    ld (ai_delay), a
    ret nz

    ld a, #_ai_delay_default
    ld (ai_delay), a

_loop:
    ld a, e_ai_state(ix)
    cp #ai_no_ai
    jr z, _no_ai_entity
_ai_entity:
    cp #ai_stand_by
    call z, ai_control_stand_by
    ld a, e_ai_state(ix)
    cp #ai_destroy
    call z, ai_control_destroy
    ld a, e_ai_state(ix)
    cp #ai_seek
    call z, ai_control_seek

_no_ai_entity:
_ent_counter = . + 1
    ld a, #0
    dec a
    ret z

    ld (_ent_counter), a
    ld de, #sizeof_e
    add ix, de

    jr _loop

    end_control_update: