;;This file is part of Project Thing.
;;
;;Project Thing is free software: you can redistribute it and/or modify
;;it under the terms of the GNU General Public License as published by
;;the Free Software Foundation, either version 3 of the License, or
;;(at your option) any later version.
;;
;;Project Thing is distributed in the hope that it will be useful,
;;but WITHOUT ANY WARRANTY; without even the implied warranty of
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;GNU General Public License for more details.
;;
;;You should have received a copy of the GNU General Public License
;;along with Project Thing.  If not, see <https://www.gnu.org/licenses/>.

;;
;;          ENTITY MANAGER
;;

.include "cpctelera.h.s"
.include "entity_manager.h.s"
.include "random_enemies.h.s"
.include "render_system.h.s"
.include "enemy.h.s"

.area _CODE

actual_level: .db #1

max_entities == 10
entity_size  == 17

_entity_num: .db 0 
_entity_pend: .dw _entity_array
_entity_array: 
DefineEntityArray _entity_array, 11  ;; Para que el borrado funcione se necisa tener un espacio de max_entities + 1 en el vector
_entity_int_pend: .dw _entity_array

increment_actual_level::
    
    ld a, (actual_level)
    inc a
    ld (actual_level), a

    ret

get_actual_level::

    ld a, (actual_level)

    ret

restart_actual_level::
    
    ld a, #1
    ld (actual_level), a

    ret

;;
;;          GET ARRAY
;;
man_entity_getArray::
    ld ix, #_entity_array
    ld a, (_entity_num)
    ret

;; Devuelve el número de entidades actuales que hay en el vector
man_entity_getNumEntitites::
    
    ld a, (_entity_num)

    ret


;; Devuelve el número de enemigos que hay en el vector
man_entity_getNumEnemies::
    
    ld a, (_entity_num) ;; Si ponemos un enemigo habrá que decrementarlo en 2
    dec a
    dec a
    dec a

    ;dec a               
    
    ret 

;; Devuelve el puntero que siempre está apuntando a la posición inicial del vector
man_entity_getInt_Pend::
    
    ld de, (_entity_int_pend)

    ret

;;
;;  SETTERS
;;

man_entity_setNumEntities::
    
    inc a
    inc a
    inc a
    ld (_entity_num), a
    
    ret

man_entity_refresh_pend::
    
    ld a, (_entity_num)
    cp #0

    ret z

    ld bc, #entity_size
    ld hl, #_entity_array

    loop:

    add hl, bc

    dec a

    jr nz, loop

    jr z, end_loop

    end_loop:

    ld (_entity_pend), hl
    
    ret

;;
;;          INIT
;;
;; Reestablece el puntero a la posición inicial del vector
;; REGISTROS ALTERADOS: A, HL
man_entity_init::
    xor a
    ld (_entity_num), a

    ld hl, #_entity_array
    ld (_entity_pend), hl

    ret

;; Hace que el registro IX apunte a la posición inicial de vector
man_pointer_init::

    ld ix, (_entity_int_pend)

    ret

;; Hace que el registro IX apunte al primer enemigo del vector
man_pointer_first_enemy::   ;; *
    
    ld de, (_entity_int_pend)
    ld bc, #entity_size

    ld h, d
    ld l, e

    add hl, bc  ;; Salto al jugador

    add hl, bc  ;; Salto al enemigo

    add hl, bc  ;; Salto a la bomba del enemigo

    ex de, hl   ;; Me coloco en el primer proyectil (si lo hay)

    ld__ixh_d
    ld__ixl_e
    
    ret

;; Pone todos los valores del vector a 0
;; REGISTROS ALTERADOS: AF, HL, DE, BC
man_reset_vector::
    
    ld a, (_entity_num)
    cp #0

    jr z, end_man_reset_vector

    ld hl, (_entity_int_pend)

    push af

    vector_loop:

    xor a
    ld (hl), a
    ld d, h
    ld e, l
    inc de
    ld bc, #entity_size

    ldir

    pop af

    dec a

    ret z

    push af

    inc hl
    dec de

    jr vector_loop

    end_vector_loop:

    end_man_reset_vector:

    ret

;;
;;          NEW ENTITY
;;
man_entity_new::
    ld hl, #_entity_num
    inc (hl)

    ld hl, (_entity_pend)
    ld d, h
    ld e, l
    ld bc, #entity_size
    add hl, bc
    ld (_entity_pend), hl

    ret

;;
;;          CREATE ENTITY
;;
;; Crear un nuevo proyectil, asignándole una de las dos posiciones (eje Y) al azar, en las que puede aparecer el proyectir   
;; REGISTROS ALTERADOS: A, HL, DE, BC, IX
man_entity_create::
    push hl
    call man_entity_new
    ;; Cambiar a la segunda posicion del vector
    ld__ixh_d               ;; / 
    ld__ixl_e               ;; |
                            ;; |
    ld bc, #entity_size     ;; | Copia la entidad (en este caso proyectil) en la posición del vector
    ;add ix, bc             ;; | que le marca el puntero
                            ;; |
    pop hl                  ;; |
    ldir                    ;; \ 

    ld a, e_state(ix)   
    cp #enemy_noIA

    jr z, random_number

    ret

    ;; Se encarga de generar un número al azar, según el número la posición de inicio
    ;; del proyectil será una u otra (hay dos posiciones, se puede ampliar a más si queremos)
    random_number:

    ld a, e_vx(ix)
    cp #0

    jp m, set_vx_proyectilderecha

    call get_proyectil_speed_left
    ld e_vx(ix), a  

    jr continue_generate_random

    set_vx_proyectilderecha:

    call get_proyectil_speed    ;; / Asigna la velocidad correspondiente al proyectil
    ld e_vx(ix), a              ;; \  

    continue_generate_random:

    call generate_random_number

    ld b, #0x99
    sub b

    jp m, minus_0

    ld e_y(ix), #0x8F     

    jp update_IX

    minus_0:

    ld e_y(ix), #0xA4    

    update_IX:

    ld bc, #entity_size
    add ix, bc

    ret

obtener_ultimo_enemigo:
    
    ld bc, #entity_size
    ld hl, #_entity_array
    ld a, (_entity_num)
    cp #0

    jr z, end_loop_obtener_ultimo_enemigo 

    loop_obtener_ultimo_enemigo:

    add hl, bc

    dec a
    cp #1

    jr z, end_loop_obtener_ultimo_enemigo

    jp loop_obtener_ultimo_enemigo

    end_loop_obtener_ultimo_enemigo:

    ret

comparar_registros:

    ld a, h
    xor d
    cp #0

    jr nz, no_equlas

    ld a, l
    xor e
    cp #0

    jr nz, no_equlas

    ld a, #0

    ret

    no_equlas:

    ld a, #1

    ret

comprobar_enemy:

    call obtener_ultimo_enemigo
    ld (actual_enemy), ix
    actual_enemy = . + 1
    ld de, #0x0000

    call comparar_registros

    ret

;;
;; DELETE ENTITY
;; 

;; Borra el enemigo deseado y reordena el vector
man_delete_entity::
    
    ld e_sprite_h(ix), #00
    ld e_sprite_l(ix), #00
    ;call rendersys_update
    call render_system_erase
    ;call man_pointer_first_enemy
    ;call man_entity_getInt_Pend ;; Cambiar
    ;ld h, d
    ;ld l, e
    call comprobar_enemy
    ;call man_entity_getNumEnemies
    cp #0

    jr z, cero_entity

    ld h, d
    ld l, e

    ld bc, #entity_size
    add hl, bc



    jr nz, no_cero

    ret

cero_entity:

    ;ld (init_position), hl

    xor a
    ld (hl), a
    inc de
    ;ld__ixh_d
    ;ld__ixl_e
    ld bc, #entity_size

    ldir

   ; init_position = . + 1
    ;ld (_entity_pend), ix

    ret


no_cero:
    
    ld (enemy_to_continue), ix

    loop_no_cero:

    ld bc, #entity_size
    
    ldir

    ;dec a
    ld__ixh_d
    ld__ixl_e
    ld (iter_enemy), ix
    ld de, (_entity_pend)
    call comparar_registros
    cp #0

    jr z, one_entity

    iter_enemy = . + 1
    ld de, #0x0000

    jr loop_no_cero

    one_entity:

    ;dec de
    ;ld h, d
    ;ld l, e
    ld (next_enemy), ix
    next_enemy = . + 1
    ld de, #0x0000
    ld h, d
    ld l ,e

    call cero_entity

    enemy_to_continue = . + 1
    ld de, #0x0000

    ret



