;;This file is part of Project Thing.
;;
;;Project Thing is free software: you can redistribute it and/or modify
;;it under the terms of the GNU General Public License as published by
;;the Free Software Foundation, either version 3 of the License, or
;;(at your option) any later version.
;;
;;Project Thing is distributed in the hope that it will be useful,
;;but WITHOUT ANY WARRANTY; without even the implied warranty of
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;GNU General Public License for more details.
;;
;;You should have received a copy of the GNU General Public License
;;along with Project Thing.  If not, see <https://www.gnu.org/licenses/>.

;;
;;RENDER SYSTEM
;;

.include "entity_manager.h.s"
.include "render_system.h.s"
.include "physics_system.h.s"
.include "input_system.h.s"
.include "assets.h.s"
.include "cpctelera.h.s"
.include "timer.h.s"

.globl cpct_scanKeyboard_f_asm
.globl cpct_waitVSYNC_asm

.include "assets/screen_start1.h.s"
.include "assets/screen_fondo.h.s"
;.include "assets/screen_fondoNegro.h.s"

screen_start = 0xC000
backgroun_color = 0x00

videoPtr: .dw #0x8000
MaxSize: .db  #0x28

pantalla: .dw #0xC000
max_pantalla:: .db #0x0A

get_maxSize::
    
    xor a
    ld a, (MaxSize)

    ret


reset_tilemap_ptr::

    ld hl, (tilemap)
    ld de, #0x40
    ex de, hl
    ld (tilemap), hl
    ret

reset_MaxSize::
    ld hl, (MaxSize)
    ld de, #0x28
    ex de, hl
    ld (MaxSize), hl
    ret

reset_posicion_pantalla_y_max_pantalla::
    ld hl, (pantalla)
    ld de, #0xC000
    ex de, hl
    ld (pantalla), hl
    ld hl, (max_pantalla)
    ld de, #0x0A
    ex de, hl
    ld (max_pantalla), hl
    ret


rendersys_Wipe::
    ;; Erase the las sprite position
    ld h, #0xC0
    ld l, #0x00
    ex de, hl
    ld  a, #backgroun_color
    ld  b, #screen_width    ;;w
    ld  c, #screen_heigth    ;;H
    call cpct_drawSolidBox_asm ;;For this first iteration Entities will not have strites and will be solid boxes instead


;;Initializes the render system by storing a pointer to the first element of the entity array
rendersys_init::
    push af

    ld de, #screen_start
    ld  b, e_y(ix)    ;;y
    ld  c, e_x(ix)    ;;x
    call cpct_getScreenPtr_asm

    ld e_lp_h(ix), h
    ld e_lp_l(ix), l

    pop af
    dec a
    ret z

    ld bc, #entity_size
    add ix, bc

    jr rendersys_init

draw_door::
    
    call get_maxSize
    cp #0

    jr nz, end_draw_door

    ;; Erase the las sprite position
    ;ld h, e_lp_h(ix)
    ;ld l, e_lp_l(ix)
    ;ex de, hl

    ;ld de, #0xC598
    ;ld  a, #backgroun_color
    ;ld  b, e_h(ix)    ;;w
    ;ld  c, e_w(ix)    ;;H
    ;call cpct_drawSolidBox_asm

    ;; Draw the new sprite and store the new position
    ld de, (pantalla)
    ld  b, #136     ;;y
    ld  c, #72        ;;x
    call cpct_getScreenPtr_asm
    ;ld e_lp_h(ix), h
    ;ld e_lp_l(ix), l
    ex de, hl
    ld hl, #_Puerta1
    ;;ld a, e_sprite(ix)
    ld  b, #32  ;;h
    ld  c, #8    ;;w
    ;;call cpct_drawSolidBox_asm ;;For this first iteration Entities will not have strites and will be solid boxes instead
    call cpct_drawSprite_asm

    ret

    end_draw_door:

    ld de, (pantalla)
    ld  b, #136   ;;y
    ld  c, #72    ;;x
    call cpct_getScreenPtr_asm

    ;; Erase the las sprite position
    ;ld h, e_lp_h(ix)
    ;ld l, e_lp_l(ix)
    ;ex de, hl
    ld de, #0xC598
    ld  a, #backgroun_color
    ld  b, #32    ;;w
    ld  c, #8    ;;H
    call cpct_drawSolidBox_asm

    ret

render_system_erase::
    
    ;; Erase the las sprite position
    ld h, e_lp_h(ix)
    ld l, e_lp_l(ix)
    ex de, hl
    ld  a, #backgroun_color
    ld  b, e_h(ix)    ;;w
    ld  c, e_w(ix)    ;;H
    call cpct_drawSolidBox_asm
    
    ret

;;INPUT
;;  IX: Pointer to first entity
;;   A: number of entities to render
rendersys_update::

_renloop:
    push af

    ld a, e_draw(ix)
    cp #on

    jr nz, next_entity

    ;; Erase the las sprite position
    ld h, e_lp_h(ix)
    ld l, e_lp_l(ix)
    ex de, hl
    ld  a, #backgroun_color
    ld  b, e_h(ix)    ;;w
    ld  c, e_w(ix)    ;;H
    call cpct_drawSolidBox_asm ;;For this first iteration Entities will not have strites and will be solid boxes instead


    ;; Draw the new sprite and store the new position
    ld de, #screen_start
    ld  b, e_y(ix)    ;;y
    ld  c, e_x(ix)    ;;x
    call cpct_getScreenPtr_asm
    ld e_lp_h(ix), h
    ld e_lp_l(ix), l
    ex de, hl
    ld  h, e_sprite_h(ix)
    ld  l, e_sprite_l(ix)
    ;;ld a, e_sprite(ix)
    ld  b, e_h(ix)    ;;w
    ld  c, e_w(ix)    ;;H
    ;;call cpct_drawSolidBox_asm ;;For this first iteration Entities will not have strites and will be solid boxes instead
    call cpct_drawSprite_asm

    next_entity:

    pop af

    dec a
    ret z

    ld bc, #entity_size
    add ix, bc
    jr _renloop

;;SWITCHES VIDEO BUFFER
map_switchBuffers::

    ld hl, #videoPtr+1
    ld a, #0x40
    xor (hl)
    ld (videoPtr+1), a
    ret

;;Returns Video Pointer in hl
map_getVideoPtr::
    ld hl, (videoPtr)
    ret

;;UPDATE MAP
map_update::
    
    call get_input_move
    cp #0

    jr z, continue_map_update

    ret

    continue_map_update:

    call cpct_scanKeyboard_f_asm
    ;call cpct_scanKeyboard_asm
    ld hl, #Key_P
    call cpct_isKeyPressed_asm
    jr nz, p_pulsada
    ld hl, #Joy0_Right
    call cpct_isKeyPressed_asm
    jr z, p_no_pulsada
    p_pulsada:
        ld  a, (MaxSize)
        cp #0
        jr z, p_no_pulsada
            ;dec a
            dec a
            ld (MaxSize), a
            call map_scrollRight
    p_no_pulsada:
        ;;If O pressed
        ;call cpct_scanKeyboard_asm
        call cpct_scanKeyboard_f_asm
        ld hl, #Key_O
        call cpct_isKeyPressed_asm
        jr nz, _Pressed_O
        ld hl, #Joy0_Left
        call cpct_isKeyPressed_asm
        jr z, _notPressed_O
        _Pressed_O:
            ld a, (MaxSize)
            cp #0x28
            jr z, _notPressed_O
                ;inc a
                inc a
                ld (MaxSize), a
                call map_scrollLeft
        _notPressed_O:
            ret

;;SCROLL RIGHT
map_scrollRight::
    ld hl, (tilemap)
    ld de, #1
    add hl, de
    ld (tilemap), hl
    ret

;;SCROLL LEFT
map_scrollLeft::
    ld hl, (tilemap)
    ld de, #-1
    add hl, de
    ld (tilemap), hl
    ret

;;DRAW THE FULL MAP
setdraw_tilemap::

    ld hl, #_mapaPruebas_pack_end 
    ld de, #decompress_buffer_end
    call cpct_zx7b_decrunch_s_asm 

    ld   bc, #0x0814
    ld   de, #_mapaPrueba_W
    ld   hl, #tileset_ptr

    call cpct_etm_setDrawTilemap4x8_ag_asm
    
    ret

;;INPUT DE: VIDEO MEMORY LOCATION WHERE TO DRAW
draw_tilemap::
    ex de, hl
    tilemap = . + 1
    ld de,  #tilemap_ptr

    call cpct_etm_drawTilemap4x8_ag_asm
    ret

draw_suelo::
   
    ld de, (pantalla)
    ld b, #168
    ld c, #00
    
    call cpct_getScreenPtr_asm

    ex de, hl
    ld hl, #_Suelo1
    ld bc, #0x2008

    call cpct_drawSprite_asm

    ld hl, (pantalla)
    ld de, #0x08
    add hl, de
    ld (pantalla), hl 

    ld a, (max_pantalla)
    dec a
    ld (max_pantalla), a



    cp #0
    jr nz, draw_suelo
    
    ;:call draw_door

    ret

draw_life::
    
    ld (name_sprite_life), hl

    ld de, (pantalla)

    call cpct_getScreenPtr_asm

     ex de, hl
    ;ld hl, #_pruebaNumero
    name_sprite_life = . + 1
    ld hl, #0x0000

    ;ld a, #0xFFFF
    ld b, #8   ;; altura
    ld c, #4    ;; Anchura
    

    ;call cpct_drawSolidBox_asm
    call cpct_drawSprite_asm

    ret

draw_timer::

    ld (name_sprite), hl

    ld de, (pantalla)
    ld b, #75
    ;ld c, #55

    call cpct_getScreenPtr_asm

     ex de, hl
    ;ld hl, #_pruebaNumero
    name_sprite = . + 1
    ld hl, #0x0000

    ;ld a, #0xFFFF
    ld b, #16   ;; altura
    ld c, #4    ;; Anchura
    

    ;call cpct_drawSolidBox_asm
    call cpct_drawSprite_asm

    ret

draw_pantalla_inicio::
    
    ld hl, #_screen_start1_end 
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm 

    ret

draw_colorfondo::

    ld hl, #_screen_fondo_end 
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm 

    ret


game_over::
    
    ;; Dibujar fondo
    call draw_colorfondo

    ld a, e_x(ix)
    cp #0x40

    jr z, no_game_over

    call draw_game_over

    jp game_over_on

    no_game_over:

    call draw_next_level

    game_over_on:

    call draw_score

    ;call draw_total_score

    loop_wait:
    call cpct_waitVSYNC_asm

    call inputsys_waitForInput
    cp #0

    jr z, loop_wait


    ret

draw_next_level:

    ld de, #0xC000
    ld b, #30
    ld c, #20
    
    call cpct_getScreenPtr_asm

    ex de, hl
    ld hl, #_LevelComplete1
    ld bc, #0x1028

    call cpct_drawSprite_asm

    ret

draw_score:
    
    ld de, #0xC000
    ld b, #77
    ld c, #0

    call cpct_getScreenPtr_asm

    ex de, hl
    ld hl, #_Score1
    ld bc, #0x1028

    call cpct_drawSprite_asm

    ld b, #80
    ld c, #52
    call get_timer_mill

    ;loop_sprite_timer:
    
    call get_sprite_name
    call draw_timer

    ld c, #56
    call get_timer_cent

    call get_sprite_name
    call draw_timer 

    ld c, #60
    call get_timer_dec

    call get_sprite_name
    call draw_timer 

    ld c, #64
    call get_timer_uni

    call get_sprite_name
    call draw_timer

    ret

;draw_total_score:
    
;    ld de, #0xC000
;    ld b, #115
;    ld c, #15

;    call cpct_getScreenPtr_asm

;    ex de, hl
;    ld hl, #_TotalScore1
;    ld bc, #0x1028

;    call cpct_drawSprite_asm

;    ret


draw_game_over::
    ld de, #0xC000
    ld b, #30
    ld c, #20
    
    call cpct_getScreenPtr_asm

    ex de, hl
    ld hl, #_GameOver1
    ld bc, #0x1028

    call cpct_drawSprite_asm
    ret